function [theta,y_cal_mean]=MIMU_calib_param_est(meas,M,C,scale_acc,scale_gyro,L,intMisalign,covW,covWFile)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%>
%>  Script that runs the IMU array calibration described in [1]
%>  The script does the following. It loads the raw data and
%>  preprocesses it. Thereafter the noise covariances of the data are
%>  estimated. Then the initial roll and pitch angles of the platform
%>  are estimated. Finally the actual calibration algorithm is
%>  processed.
%>
%>  [1] YYYY
%>
%>  @authors Isaac Skog, John-Olof Nilsson
%>  @copyright Copyright (c) 2013 OpenShoe, ISC License (open source)
%>
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%% Settings and memory allocation

global y_meas;
global cholQinv;
y_meas=zeros(M,3,L);                        % Allocate memory for the measurements
cholQinv=zeros(M,3,3,L);                    % Allocate memory for the noise covariance matrices
theta=zeros(12,M);

y_cal_mean=zeros(M,3,L);

gyro_mean = zeros(M,3);

%% Load and preprocess the data

% Loop through the data recorded at the L different orientations and
% take the arithmetic mean of the data.  Estimate the noise covariance
% also.
if covW
    load(covWFile);
end

for l=1:L
    % Convert to double precision
    inertial_data=double(meas{l});
    
    % Store the data from the M IMUs into the global variable y_meas
    for m=1:M
        
        % Correct for the mounting direction of the different IMUs
        R = reshape(C(m,1:9),3,3);
        
        % Calculate the mean of the accelerometer readings of the m:th IMU
        % at the l:th orientation
        tmp=scale_acc*R*inertial_data(6*(m-1)+1:6*(m-1)+3,:);
        y_meas(m,:,l)=mean(tmp,2);
        gyro_mean(m,:)=gyro_mean(m,:)+mean(scale_gyro*R*inertial_data(6*(m-1)+4:6*(m-1)+6,:),2)'/L;
        
        if covW
            cholQinv(m,:,:,l)=cov_inert_data(m*6-5:m*6-3,:);
        else
            % Estimate the noise covariance of the m:th IMU at the l:th orientation
            cholQinv(m,:,:,l)=chol(inv(cov(tmp')));
        end
        
    end
end
% Put first (reference) IMU data last
tmp = y_meas(M,:,:);
y_meas(M,:,:) = y_meas(1,:,:);
y_meas(1,:,:) = tmp;
tmp = cholQinv(M,:,:,:);
cholQinv(M,:,:,:) = cholQinv(1,:,:,:);
cholQinv(1,:,:,:) = tmp;

%% Calculate the initial estimates for the roll and pitch
tmp=zeros(2,L);
for l=1:L
    
    % Calculate the mean over all IMUs.
    f=mean(y_meas(:,:,l))';
    
    % Roll
    tmp(1,l)=atan2(f(2),f(3));
    
    % Pitch
    tmp(2,l)=atan2(-f(1),sqrt(f(2)^2+f(3)^2));
end

%% Run estimation of the accelerometer scalefactors, biases, and IMU misalignment

if intMisalign
    % Define the initial state estimate
    theta0=[reshape(tmp,2*L,1); repmat([1 1 1  0 0 0  0 0 0  0 0 0]',M,1)];
    % Remove alignment parameters of last IMU
    theta0=theta0(1:end-3);
    % Run the optimazation
    tmp=lsqnonlin(@costFunctionArrayInternalMisalignment,theta0);
    % Trow away the nuisance parameters, reinsert zeros for alignment of
    % last IMU, and reshape parameters into original form
    theta(:,:)=reshape([tmp(2*L+1:end); zeros(3,1)],12,M);
else
    theta0=[reshape(tmp,2*L,1); repmat([1 1 1  0 0 0  0 0 0]',M,1)];
    theta0=theta0(1:end-3);
    % Run the optimazation
    tmp=lsqnonlin(@costFunctionArray,theta0);
    % Trow away the nuisance parameters, reinsert zeros for alignment of
    % last IMU, and reshape parameters into original form
    tmp=reshape([tmp(2*L+1:end); zeros(3,1)],9,M);
    % Insert zeros for internal misalignment
    theta(:,:)=[tmp(1:6,:); zeros(3,M); tmp(7:9,:)];
end

%% Calibrated the (time) mean value of the data at each orientation
for m=1:M
    theta_tmp=squeeze(theta(:,m));
    
    % Set the calibration parameters
    K=diag(theta_tmp(1:3));
    S=eye(3);
    S(1,2)=-theta_tmp(7);
    S(1,3)=theta_tmp(8);
    S(2,3)=-theta_tmp(9);
    b=theta_tmp(4:6);
    b=b(:);
    E=eye(3);
    E(1,2)=theta_tmp(12);
    E(1,3)=-theta_tmp(11);
    E(2,1)=-theta_tmp(12);
    E(2,3)=theta_tmp(10);
    E(3,1)=theta_tmp(11);
    E(3,2)=-theta_tmp(10);
    
    % Calibrate the accelerometers and the misalignment
    for l=1:L
        tmp=squeeze(y_meas(m,:,l))' -b;
        y_cal_mean(m,:,l)=(K*S*E)\tmp;
    end
end

% Switch back order of data (see switch after loading of data)
theta=theta';
tmp = theta(1,:);
theta(1,:) = theta(M,:);
theta(M,:) = tmp;

% Append gyro mean (biases) values
theta = [theta gyro_mean];

tmp = y_cal_mean(1,:,:);
y_cal_mean(1,:,:) = y_cal_mean(M,:,:);
y_cal_mean(M,:,:) = tmp;

end





