
% Script for collecting inertial calibration data in combination with a
% calibration body providing a number of orientation for static placement
% of the IMU array. Follow the collection, the calibration parameters
% (scale factor,array alignment,internal alignment,bias) are estimated and
% a calibration matrix for the whole array is generated. Finally,
% calibration code is generated and the recorded data is displayed for
% inspection.
% Authors: John-Olof Nilsson, Isaac Skog

% Prep workspace
clear all, close all, clc
format compact
global g_local;

% Parameters
internalMisalignment = true;    % Model internal misalignment or not
covarianceWeighting = false;
covarianceWeightingFile = './Calibration_MIMU4444BT/static_20150224_225453_1';
target_board = 'MIMU22BT';      % MIMU22BT, MIMU3333, MIMU4444, MIMU4444BT
g_local = gravity(59,10);       % Stockholm (latitude, altitude)
com_port = 'COM76';             % Com-port address of your module

%System settings
scale_acc  = 1/2048*9.80665;
scale_gyro = 1/16.4;
calib_gain = 2^15;

% Open serial port
com = serial(com_port,'InputBufferSize',100000);
fopen(com);

% Load settings for board
[nr_imus,C_nom,command] = board_settings(target_board);

% Calculate desired time on each side for pacing sound
desired_nr_samples = 600;               % Approx. number of samples per side
freq = 1000*2^(1-rem(command(6),64));   % Sampling frequency
time_side = desired_nr_samples/freq;    % Time on each side

disp(' ');
disp('Script for logging calibration data sets with wireless MIMU platform and icosahedron rig.');
disp('Please verify that the platform is warmed up.');
disp(' ');

disp('Let the calibration body sit stationary for a few seconds on each side.');
disp('(A pacing beep will indicate how long you should leave the body sitting)');
disp('Press abort on the appearing figure when done.');
disp('Data will be displayed for inspection afterward.');
disp(' ');

input('Press any key to start.','s');

% Prepare figure with pushbutton such that logging can be aborted
figure(1),clf
abort = 0;
uicontrol('style','push','string','Stop data logging','callback','abort=1;');
drawnow

% Make sure data is read from the IMUs
fwrite(com,[48 19 0 0 67],'uint8');
fread(com,4,'uint8');

% Tell OpenShoe to output inertial data
fwrite(com,command,'uint8');
fread(com,4,'uint8');

% Open binary file for saving inertial data
filename = 'imu_data.bin';
file = fopen(filename,'w');

% Logg data until pushbutton is pressed
time_for_next_beep = cputime + time_side;
while abort==0
    if com.BytesAvailable>0
        fwrite(file,fread(com,com.BytesAvailable,'uint8'),'uint8');
    end
    drawnow
    if time_for_next_beep<cputime
        beep;
        time_for_next_beep = cputime + time_side;
    end
end

% Stop output and close file
fwrite(com,[34 0 34],'uint8');
fclose(com);
fclose(file);

% Parse recorded data
[inertial_data,time_stamps,raw_data]=mimu_parse_bin(filename,uint8(nr_imus));
[calibration_measurements,nr_sides] = extract_stationary_segments(inertial_data,C_nom);
delete(filename);

% Check that we have sufficient number of sides
fprintf('Number of detected (sufficiently stationary) sides: %i\n\n',nr_sides);
if nr_sides<12
    disp('To few sides recorded. Aborting.')
    return
end

% Estimate and save the calibration parameters
disp(' ');
disp('Estimating calibration parameters (this may take a while)');
[theta,y_cal_mean]=MIMU_calib_param_est(calibration_measurements,nr_imus,C_nom,scale_acc,scale_gyro,nr_sides,internalMisalignment,covarianceWeighting,covarianceWeightingFile);
disp('Calibration result:');
disp(theta);

% Calculate and generate code for calibration matrix for
disp(' ');
disp('Generating calibration code files');
[C_int,b_acc_int,b_gyro_int] = calibration_matrix(theta,1/scale_acc,1/scale_gyro,C_nom,calib_gain);
if covarianceWeighting
    load(covarianceWeightingFile,'inertial_data2');
    W = covariance_weighting(inertial_data2,double(C_int),nr_imus);
    clear('inertial_data2');
    [C_int,b_acc_int,b_gyro_int] = calibration_matrix_with_cov_weighting(theta,1/scale_acc,1/scale_gyro,C_nom,calib_gain,W);
end
calib_filename = ['user_calibration_' target_board '.h'];
code_generation(C_int,b_acc_int,b_gyro_int,nr_imus,calib_filename);
disp('Calibration matrix:');
disp(b_acc_int);
disp(b_gyro_int);
disp(C_int);

% Plot calibration data (with and without compensation for calibration)
disp('Plotting calibration data');
MIMU_plot_calib(calibration_measurements,nr_imus,C_int,C_nom,theta,scale_acc,scale_gyro,calib_gain,b_acc_int,b_gyro_int,nr_sides);

% Save workspace
filename = sprintf('./calibration_%02i%02i%02i_%02i%02i%02i',fix(clock()));
save(filename);
disp(['The data and the result are saved in ' filename '.mat']);
disp(['The configuration code file ' calib_filename ' has been generated']);


