
% Script for collecting inertial data for Allan deviation analysis and
% covariance weighting of calibration matrix. This script will collect the
% data, save it for later (calibration parameter estimation and calibration
% matrix generation) usage, and calculate and display the Allan variance.
% Authors: John-Olof Nilsson

% Prep workspace
clear all, close all, clc
format compact
global g_local;

% Parameters
target_board = 'MIMU4444BT';      % MIMU22BT, MIMU3333, MIMU4444, MIMU4444BT
com_port = 'COM6';             % Com-port address of your module

%System settings
scale_acc  = 1/2048*9.80665;
scale_gyro = 1/16.4;
calib_gain = 2^15;

% Open serial port
com = serial(com_port,'InputBufferSize',1000000);
fopen(com);

% Load settings for board
[nr_imus,C_nom,command] = board_settings(target_board);

disp(' ');
disp('Script for logging for Allan deviation analysis and covariance weighting.');
disp('Please verify that the platform is warmed up.');
disp(' ');

disp('Let the platform sit still and log data for at least 10min.');
disp('Press abort on the appearing figure when done.');
disp('Data will be displayed for inspection afterward.');
disp(' ');

input('Press any key to start.','s');

% Prepare figure with pushbutton such that logging can be aborted
figure(1),clf
abort = 0;
uicontrol('style','push','string','Stop data logging','callback','abort=1;');
drawnow

% Make sure data is read from the IMUs
fwrite(com,[48 19 0 0 67],'uint8');
fread(com,4,'uint8');

%command = [40 0 0 0 15 1+64 0 56+64];
command = [40 255 255 255 255 1+64 4 37+64];
% Tell OpenShoe to output inertial data
fwrite(com,command,'uint8');
fread(com,4,'uint8');

% Open binary file for saving inertial data
filename = 'imu_data.bin';
file = fopen(filename,'w');

% Logg data until pushbutton is pressed
while abort==0
    if com.BytesAvailable>0
        fwrite(file,fread(com,com.BytesAvailable,'uint8'),'uint8');
    end
    drawnow
end

% Stop output and close file
fwrite(com,[34 0 34],'uint8');
fclose(com);
fclose(file);

% Parse recorded data
[inertial_data2,time_stamps,raw_data]=mimu_parse_bin(filename,uint8(nr_imus));
%delete(filename);
disp(' ')

%% Display recorded data
inertial_data_SI = bsxfun(@times,repmat([scale_acc scale_acc scale_acc scale_gyro scale_gyro scale_gyro]',nr_imus,1),double(inertial_data2));
% inertial_data_SI = double(inertial_data2);
scale_acc  = 1/2048*9.80665;
scale_gyro = 1/16.4;
figure(1),clf, hold on
for i=0:nr_imus-1
    plot(inertial_data_SI(i*6+1,:)','b-')
    plot(inertial_data_SI(i*6+2,:)','g-')
    plot(inertial_data_SI(i*6+3,:)','r-')
end
grid on
title('Accelerometer readings');
xlabel('sample number')
ylabel('a [m/s^2]');
figure(2),clf, hold on
for i=0:nr_imus-1
    plot(inertial_data_SI(i*6+4,:)','b-')
    plot(inertial_data_SI(i*6+5,:)','g-')
    plot(inertial_data_SI(i*6+6,:)','r-')
end
grid on
title('Gyroscope readings');
xlabel('sample number')
ylabel('\omega [deg/s]');
figure(3),clf, hold on
subplot(2,1,1);
plot(double(time_stamps)'/64e6,'b-');
grid on
title('Time stamps');
xlabel('sample number')
ylabel('[s]');
subplot(2,1,2);
dt = diff(double(time_stamps)');
for i=1:numel(dt)
    if dt(i)<0
        dt(i) = dt(i)+2^32;
    end
end
plot(dt/64e6,'b-');
grid on
title('Time differentials');
xlabel('sample number')
ylabel('[s]');

%%

% Calculate covariance of measurements of individual IMUs
cov_inert_data = zeros(nr_imus*6,3);
for i=1:nr_imus
    cov_inert_data(i*6-5:i*6-3,:) = cov(double(inertial_data2(i*6-5:i*6-3,:))');
    cov_inert_data(i*6-2:i*6-0,:) = cov(double(inertial_data2(i*6-2:i*6-0,:))');
end

% Save covariance and data
filename = sprintf('./static_%02i%02i%02i_%02i%02i%02i',fix(clock()));
save(filename,'cov_inert_data','inertial_data2');

%% Allan deviation analysis

freq = 1000*2^(1-rem(command(6),64));
tau = logspace(ceil(log10(1/freq)+1),floor(log10(size(inertial_data2,2)/freq/4)),(floor(log10(size(inertial_data2,2)/freq/4))-ceil(log10(1/freq)))*10);

astd = allan_variance(inertial_data_SI,freq,tau);
astdm = allan_variance(repmat(eye(6)/nr_imus,1,nr_imus)*inertial_data_SI,freq,tau);

figure(10), clf, set(gca,'XScale','log','YScale','log'), axis([tau(1) tau(end) 10^-1 10^2]), set(gca,'ColorOrder',[0 0 1;0 1 0;1 0 0]), grid on, hold on
title('Accelerometer -- Allan variance'); xlabel('Cluster time [s]'); ylabel('Allan standard deviation [m/s/h]')
loglog(tau,astd(logical(repmat([1 1 1 0 0 0]',nr_imus,1)),:)*3600);
loglog(tau,astdm(1:3,:)*3600,'LineWidth',2);

figure(11), clf, set(gca,'XScale','log','YScale','log'), axis([tau(1) tau(end) 10^0 10^3]), set(gca,'ColorOrder',[0 0 1;0 1 0;1 0 0]), grid on, hold on
title('Gyroscope -- Allan variance'); xlabel('Cluster time [s]'); ylabel('Allan standard deviation [/h]')
loglog(tau,astd(logical(repmat([0 0 0 1 1 1]',nr_imus,1)),:)*3600);
loglog(tau,astdm(4:6,:)*3600,'LineWidth',2);

% Vikta med covariansen
astdw = zeros(6,numel(tau));
for i=1:6
    select = zeros(6,1);    select(i)=1;
    f_x = inertial_data_SI(logical(repmat(select,nr_imus,1)),:);
    tmp = inv(cov(f_x'));
    wf_x = sum(tmp)/sum(sum(tmp));
    f_mean = wf_x*f_x;
    astdw(i,:) = allan_variance(f_mean,freq,tau);
end
figure(10)
loglog(tau,astdw(1:3,:)*3600,':','LineWidth',2);
figure(11)
loglog(tau,astdw(4:6,:)*3600,':','LineWidth',2);

% load('calibration_20150217_201331');
% 
% W = covariance_weighting(inertial_data2,double(C_int),nr_imus);
% [C_int,b_acc_int,b_gyro_int] = calibration_matrix_with_cov_weighting(theta,1/scale_acc,1/scale_gyro,C_nom,calib_gain,W);
% calib_filename = ['user_calibration_' target_board '.h'];
% code_generation(C_int,b_acc_int,b_gyro_int,nr_imus,calib_filename);
% MIMU_plot_calib(calibration_measurements,nr_imus,C_int,C_nom,theta,scale_acc,scale_gyro,calib_gain,b_acc_int,b_gyro_int,nr_sides);



